# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: artem.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='artem.proto',
  package='',
  syntax='proto3',
  serialized_pb=_b('\n\x0b\x61rtem.proto\"\x90\x02\n\nProtoArtem\x12\x15\n\rglobal_admins\x18\x01 \x03(\x05\x12\x14\n\x0cglobal_names\x18\x02 \x03(\t\x12\x18\n\x10pooling_interval\x18\x03 \x01(\x02\x12/\n\x0e\x64ialog_threads\x18\x04 \x03(\x0b\x32\x17.ProtoArtem.ProtoThread\x1a\x89\x01\n\x0bProtoThread\x12\x0f\n\x07some_id\x18\x01 \x01(\x05\x12\x1e\n\x16\x64iscourse_interval_max\x18\x02 \x01(\x05\x12\r\n\x05names\x18\x03 \x03(\t\x12\x0e\n\x06\x61\x64mins\x18\x04 \x03(\x05\x12\x18\n\x10session_duration\x18\x05 \x01(\x02\x12\x10\n\x08sessions\x18\x06 \x01(\x08\x62\x06proto3')
)




_PROTOARTEM_PROTOTHREAD = _descriptor.Descriptor(
  name='ProtoThread',
  full_name='ProtoArtem.ProtoThread',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='some_id', full_name='ProtoArtem.ProtoThread.some_id', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='discourse_interval_max', full_name='ProtoArtem.ProtoThread.discourse_interval_max', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='names', full_name='ProtoArtem.ProtoThread.names', index=2,
      number=3, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='admins', full_name='ProtoArtem.ProtoThread.admins', index=3,
      number=4, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='session_duration', full_name='ProtoArtem.ProtoThread.session_duration', index=4,
      number=5, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sessions', full_name='ProtoArtem.ProtoThread.sessions', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=151,
  serialized_end=288,
)

_PROTOARTEM = _descriptor.Descriptor(
  name='ProtoArtem',
  full_name='ProtoArtem',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='global_admins', full_name='ProtoArtem.global_admins', index=0,
      number=1, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='global_names', full_name='ProtoArtem.global_names', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='pooling_interval', full_name='ProtoArtem.pooling_interval', index=2,
      number=3, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='dialog_threads', full_name='ProtoArtem.dialog_threads', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_PROTOARTEM_PROTOTHREAD, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=16,
  serialized_end=288,
)

_PROTOARTEM_PROTOTHREAD.containing_type = _PROTOARTEM
_PROTOARTEM.fields_by_name['dialog_threads'].message_type = _PROTOARTEM_PROTOTHREAD
DESCRIPTOR.message_types_by_name['ProtoArtem'] = _PROTOARTEM
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ProtoArtem = _reflection.GeneratedProtocolMessageType('ProtoArtem', (_message.Message,), dict(

  ProtoThread = _reflection.GeneratedProtocolMessageType('ProtoThread', (_message.Message,), dict(
    DESCRIPTOR = _PROTOARTEM_PROTOTHREAD,
    __module__ = 'artem_pb2'
    # @@protoc_insertion_point(class_scope:ProtoArtem.ProtoThread)
    ))
  ,
  DESCRIPTOR = _PROTOARTEM,
  __module__ = 'artem_pb2'
  # @@protoc_insertion_point(class_scope:ProtoArtem)
  ))
_sym_db.RegisterMessage(ProtoArtem)
_sym_db.RegisterMessage(ProtoArtem.ProtoThread)


# @@protoc_insertion_point(module_scope)



import enum

from .scenario import *

def run_scen(scen):
    if not isinstance(scen, Scenario):
        return None
    new_answers = scen.respond()
    if not scen.answer:
        if not new_answers:
            return [m('')]
        elif type(new_answers) == str:
            new_answers = [m(new_answers)]
        for answer in new_answers:
            if ('message' not in answer or 
                    'sleep' not in answer or 'attach' not in answer):
                return [m('')]
        if scen.message:
            scen.messages_history.append(scen.message)
        return new_answers
    else:
        if not new_answers:
            return [m(scen.answer)]
        elif type(new_answers) == str:
            if new_answers == '':
                new_answers = [m(scen.answer)]
            else:
                new_answers = [m(new_answers)]
            return new_answer
        else:
            try:
                for answer in new_answers:
                    if ('message' not in answer
                            or 'sleep' not in answer or 
                            'attach' not in answer):
                        return [m(scen.answer)]
            except Exception:
                return [m(scen.answer)]
            return new_answers

class EventMetaclass(enum.EnumMeta):

    def __getitem__(self, key):
        if isinstance(key, str):
            key = key.upper()
        for item in self:
            if item.name == key:
                return item
        return None
        return super().__getitem__(key)


class Event(enum.Enum, metaclass=EventMetaclass):
    START = 1
    ADDFRIEND = 2
    ANSWER = 3
    POSTPROC = 4
    DISCOURSE = 5



class Lib(object):


    class Scenarios(object):


        class ScenariosIterator(object):

            def __init__(self, list_):
                self._list = list_
                self._cursor = 0
        
            def __next__(self):
                if self._cursor >= len(self._list):
                    raise StopIteration
                else:
                    result = self._list[self._cursor]
                    self._cursor += 1
                    return result


        class ScenInfo(object):
            
            def __init__(self, scen_class_type, priority):
                self.scn_type = scen_class_type
                self._priority = priority
                self.status = True

            @property
            def scn_type(self):
                return self._scn_type

            @scn_type.setter
            def scn_type(self, value):
                self._scn_type = value

            @property
            def priority(self):
                return self._priority

            @property
            def status(self):
                return self._status

            @status.setter
            def status(self, value):
                if isinstance(value, bool):
                    self._status = value


        def __init__(self, event):
            self.event = event
            self._scenarios = []

        def add(self, scn_type, priority):
            self._scenarios.append(self.ScenInfo(scn_type, priority))
            self._scenarios.sort(key=lambda s: s.priority, reverse=True)

        def such_scen(self, scn_name):
            return find_element(
                    self._scenarios, 
                    lambda s: s.scn_type.__name__.lower() == scn_name
                    )

        def get_status(self, scn_name):
            return self.such_scen(scn_name).status
        
        def set_status(self, scn_name, status):
            scn = self.such_scen(scn_name)
            if scn:
                scn.status = status
        
        def __iter__(self):
            return self.ScenariosIterator(self._scenarios)
        
        def __len__(self):
            return len(self._scenarios)

        def __getitem__(self, key):
            if key is None:
                raise KeyError
            elif not isinstance(key, int):
                raise TypeError
            else:
                return self._scenarios[key]


    def __init__(self):
        self._all_scenarios = []
        for event in Event:
            self._all_scenarios.append(self.Scenarios(event))

    def _get_scenarios(self, event):
        if isinstance(event, str):
            event = Event[event.upper()]
        return find_element(self._all_scenarios, lambda s: s.event == event)

    def get_sceninfo(self, event, scn_name):
        return self._get_scenarios(event).such_scen(scn_name.lower())

    def get_status(self, event, scn_name):
        return self._get_scenarios(event).get_status(scn_name.lower())

    def set_status(self, event, scn_name, status):
        self._get_scenarios(event).set_status(scn_name.lower(), status)

    def add(self, event, scn_type, priority):
        self._get_scenarios(event).add(scn_type, priority)

    def exists(self, event, scn_name):
        return (True 
                if self._get_scenarios(event).such_scen(scn_name)
                else False)
    
    def __getitem__(self, key):
        if isinstance(key, str):
            key = Event[key]
        if not isinstance(key, Event):
            raise TypeError
        else:
            return self._get_scenarios(key)


class Interlocutor(object):

    def __init__(self, id, first_name, last_name):
        self._id = id
        self._first_name = first_name
        self._last_name = last_name

    @property
    def id(self):
        return self._id

    @id.setter
    def id(self, value):
        self._id = value

    @property
    def first_name(self):
        return self._first_name

    @first_name.setter
    def first_name(self, value):
        self._first_name = value

    @property
    def last_name(self):
        return self._last_name

    @last_name.setter
    def last_name(self, value):
        self._last_name = value


class Envelope(object):

    def __init__(self, event, sender_id, message):
        self._message = message
        self._sender_id = sender_id
        self._event = event
    
    @property
    def message(self):
        return self._message

    @message.setter
    def message(self, value):
        self._message = value

    @property
    def sender_id(self):
        return self._sender_id

    @sender_id.setter
    def sender_id(self, value):
        self._sender_id = value
        
    @property
    def event(self):
        return self._event

    @event.setter
    def event(self, value):
        self._event = value


class ToSend(object):

    def __init__(self, some_id, message, attachment):
        self.id = some_id
        self.message = message
        self.attach = attachment

    @property
    def id(self):
        return self._id

    @id.setter
    def id(self, value):
        self._id = value

    @property
    def message(self):
        return self._message

    @message.setter
    def message(self, value):
        self._message = value

    @property
    def attach(self):
        return self._attach

    @attach.setter
    def attach(self, value):
        self._attach = value


class Wrap(object):

    @property
    def val(self):
        return self._val

    @val.setter
    def val(self, value):
        self._val = value


import random

class Scenario(object):

    # list of others.interlocutor
    interlocutors = None
    # list of message
    messages_history = None
    # list of Artem names
    names = None

    max_replicas = 5
    max_idle_time = 20
    with_all = False

    message = None
    i_sender = None
    is_personal = None
    answer = None
        
    # return boolean
    @staticmethod
    def suitable(message, i_sender, interlocutors, is_personal, name):
        return True

    # function(self, message)
    # return answers [{message: 'str', sleep: 1.0, attach: 'photo'}]
    respond = lambda self: [u'Тест сценария ' + self.__class__.__name__]

def wrapup(func):
    def wrap(self):
        ret = func()
        self.respond = None
        return ret
    return wrap

def m(text, sleep=0.0, attach=None):
    if sleep == 0.0:
        sleep = 1.0 + 2 * round(random.random(), 3)
    return {'message': text, 'sleep': sleep, 'attach': attach}    

def set_env(scen, interlocutors, names):
    scen.messages_history = []
    scen.interlocutors = interlocutors
    scen.names = names

def update_env(scen, message, i_sender, is_personal=True, answer=None):
    scen.message = message
    scen.i_sender = i_sender
    scen.is_personal = is_personal
    scen.answer = answer

def find_element(enumerate_, predicate):
    ret_value = None
    for item in enumerate_:
        if predicate(item):
            ret_value = item
            break
    return ret_value

def remove_name(message, name):
    if not message or not name:
        return None
    message = message.replace(name, '', 1)
    if len(message) != 0:
        message.lstrip()
        if message[0] == ',' or message[0] == '.' or message == '!':
            message = message[1:len(message)]
        message.lstrip()
    return message



import threading
import random
import traceback
import datetime
import queue
import copy
import time

from .scenario import *
from .others import *

DEFAULT_SESSION_DURATION = 30.0
DEFAULT_ENABLED_SESSION = True
DEFAULT_DISCOURSE_INTERVAL_MAX = 86400

class DialogThread(threading.Thread):

    # {"party"(unique): Scenario class object}    
    _run_scen = []

    # list of pair [["party"(non unique): Postproc scenario class object]]
    _run_post_scen = []

    # {"user_id": Thread class object}
    _sessions = {}

    def __init__(self, some_id, postback_queue, lib, interlocutors, logger,
                 start, global_names, global_admins):

        threading.Thread.__init__(self)
        self.daemon = True

        self.some_id = some_id
        self._postback_queue = postback_queue
        self.queue = queue.Queue()
        self.interlocutors = interlocutors
        self._logger = logger
        self.enabled_session = Wrap()
        self.enabled_session.val = DEFAULT_ENABLED_SESSION
        self.session_duration = Wrap()
        self.session_duration.val = DEFAULT_SESSION_DURATION
        self.discourse_interval_max = Wrap()
        self.discourse_interval_max.val = DEFAULT_DISCOURSE_INTERVAL_MAX
        self._start = start
        self._run_scen = []
        self._run_post_scen = []
        self._sessions = {}
        self.local_names = []
        self.local_admins = []
        self.lib = copy.deepcopy(lib)

        self._global_names = global_names
        self._global_admins = global_admins
        self._global_lib = lib

    def restore(self, sessions, duration, discourse,
                local_names, local_admins):
        self.enabled_session.val = sessions
        self.session_duration.val = duration
        self.discourse_interval_max.val = discourse
        if 'мот global' in local_names:
            local_names.remove('мот global')
        self.local_names = local_names
        self.local_admins = local_admins

    def run(self):
        try:
            if self._start:
                self._run(Event.START, msg=None, one=False)
            self._activate_discourse()
        except:
            self._logger.error(traceback.format_exc())

        while True:
            try:
                msg = self.queue.get()
                if msg.event == Event.ANSWER:
                    self._run(Event.ANSWER, msg)
                elif msg.event == Event.ADDFRIEND:
                    self._run(Event.ADDFRIEND, msg)
            except:
                self._logger.error(traceback.format_exc())

    # FEROCITY or 'лютая дичь'
    def _run(self, event, msg, one=True):

        message = msg.message if msg else None
        sender_id = msg.sender_id if msg else None
        is_personal = True
        name = None
        if len(self.interlocutors) > 1:
            if event == Event.ANSWER:
                if sender_id in self._sessions and self.enabled_session.val:
                    self._sessions[sender_id].cancel()
                    self._sessions[sender_id] = threading.Timer(
                            self.session_duration.val,
                            self._drop_session, {sender_id})
                    self._sessions[sender_id].start()
                else: 
                    name = find_element(
                            self._global_names,
                            lambda nam: message.startswith(nam))
                    if not name:
                        name = find_element(
                                self.local_names,
                                lambda nam: message.startswith(nam))
                    if name:
                        if self.enabled_session.val:
                            self._sessions[sender_id] = threading.Timer(
                                    self.session_duration.val, 
                                    self._drop_session, {sender_id})
                            self._sessions[sender_id].start()
                    else:
                        is_personal = False

        def answer_handler(answers):
            for ans in answers:
                if ans['message'] == '':
                    continue
                else:
                    time.sleep(ans['sleep'])
                    postproc_answers = self._find_answer(
                        Event.POSTPROC,
                        sender_id, message, is_personal, name,
                        lambda a: (a, len(a) > 1),
                        ans['message'], False
                        )
                    attach = None
                    if ans['attach']:
                        attach = ans['attach']
                    #print(postproc_answers)
                    for i in range(0, len(postproc_answers)):
                        send = ToSend(
                                self.some_id,
                                postproc_answers[i]['message'],
                                postproc_answers[i]['attach'])
                        if i == 0: 
                            if not postproc_answers[i]['attach']:
                                send.attach = attach
                        else:
                            time.sleep(postproc_answers[i]['sleep'])
                        self._postback_queue.put(send)
            return answers, False

        self._find_answer(event, sender_id, message, is_personal,
                          name, answer_handler)

    def _find_answer(self, event, sender_id, message, is_personal,
                     name, handler, answer=None, one=True):

        i_sender = find_element(
                self.interlocutors,
                lambda i: i.id == sender_id
                )
        if event == Event.POSTPROC:
            run = self._run_post_scen
        else:
            run = self._run_scen

        i = 0
        while i != len(run):
            if run[i][1].max_idle_time:
                left_time = (datetime.datetime.now() - run[i][1].time).seconds / 60
                if left_time >= run[i][1].max_idle_time:
                    run.remove(run[i])
                    i -= 1
            i += 1

        index = 0
        ret_answers = []
        ret_answers.append(m(answer))
        while index != len(run) + len(self.lib[event]):
            scen = None
            if index < len(run):
                item = run[index]
                if item[0] == 'all' or item[0] == sender_id:
                    scen = item[1]
                else:
                    index += 1

            else:
                scen_info = self.lib[event][index - len(run)]
                is_suit = scen_info.scn_type.suitable(
                        message, i_sender, self.interlocutors,
                        is_personal, name
                        )
                find_run = find_element(
                        run, lambda item: 
                            type(item[1]) == scen_info.scn_type and
                            item[0] == ('all' or sender_id)
                        )
                index += 1
                if (is_suit and not find_run and scen_info.status and
                        self._global_lib[event][index - len(run) - 1].status):
                    # print('run scenario ' + str(scen_info.scn_type))
                    scen = scen_info.scn_type()
                    scen.replic_count = 0
                    set_env(scen, self.interlocutors,
                            self._global_names + self.local_names)
                    new_id = ('all' 
                            if not sender_id or scen.with_all
                            else sender_id
                            )
                    run.append([new_id, scen])
            if not scen:
                continue

            index += 1
            answers = None
            scen.replic_count += 1
            update_env(scen, message, i_sender,
                       is_personal, answer)
            try:
                answers = run_scen(scen)
            except:
                self._logger.error(traceback.format_exc())
            scen.time = datetime.datetime.now()
            #print(answers)

            if answers and [ans for ans in answers if ans['message'] != '']:
                success = True
                ret_answers, end = handler(answers)
            else:
                success, end = False, False 

            if (not answers or not scen.respond or 
                    scen.replic_count == scen.max_replicas):

                run.remove(find_element(
                            run, lambda item: item[1] == scen)
                        )
                index -= 1
                if not answers:
                    raise Exception('Scenario ' +
                            type(scen).__name__ + ' return not that.')
            if (success and one) or end:
                break

        return ret_answers


    def _activate_discourse(self):
        rnd_time = random.randint(10, self.discourse_interval_max.val)
        #print('Random time: ' + str(rnd_time) + ' s.')
        timer = threading.Timer(rnd_time, self._discourse, {})
        timer.start()

    def _discourse(self):
        try:
            self._run(Event.DISCOURSE, None)
        except:
            self._logger.error(traceback.format_exc())
        finally:
            self._activate_discourse()

    def _drop_session(self, user_id):
        try:
            if user_id in self._sessions:
                del self._sessions[user_id]
        except:
            self._logger.error(traceback.format_exc())

    # duct tape
    def stop_scenario(self, event, scen_name):
        if event == Event.POSTPROC:
            el = find_element(
                    self._run_post_scen,
                    lambda el: type(el[1]).__name__.lower() == scen_name)
            while el:
                self._run_post_scen.remove(el)
                el = find_element(
                        self._run_post_scen,
                        lambda el: type(el[1]).__name__.lower() == scen_name)
        else:
            keys_to_del = []
            for key in self._run_scen:
                if type(self._run_scen[key])._name_.lower() == scen_name:
                    keys_to_del.append(key)
            for key in keys_to_del:
                del self._run_scen[key]



import re
import enum
import types
import traceback

from . import artem_core
from .scenario import find_element

class CommandType(enum.Enum):
    INFO = 1
    ADD_DEL = 2
    ON_OFF = 3
    SET = 4

class ArgType(enum.Enum):
    WORD = 1
    WORDS = 2
    INTEGER = 3
    FLOAT = 4
    STRING = 5
    ON_OFF = 6
    MESSAGE = 7

class ArgRole(enum.Enum):
    VALUE = 1
    KEY = 2
    ARG = 3
    APPEND = 4
    FUNC_ARG = 5

class AdminClass(enum.Enum):
    NONE = 1
    LOCAL = 2
    GLOBAL = 3

class Control(object):

    class Command(object):

        class Action(object):

            command_pattern = {
                CommandType.INFO: '',
                CommandType.ADD_DEL: ' (add|del)',
                CommandType.ON_OFF: ' (on|off)',
                CommandType.SET: ' set'
            }

            arg_pattern = {
                ArgType.WORD: ' ([_a-zа-я]){1,20}',
                ArgType.WORDS: ' ([_ a-zа-я](?!global)){1,40}',
                ArgType.INTEGER: ' ([0-9]){1,10}',
                ArgType.FLOAT: ' ([0-9]){1,5}(.([0-9]){1,10}){0,1}',
                ArgType.STRING: ' ([_ a-zа-я0-9](?!global)){1,20}',
                ArgType.ON_OFF: '',
                ArgType.MESSAGE: '.+'
            }

            def __init__(self, command, cmd_type, need_admin, args, func, glob):
                self._command = command
                self._cmd_type = cmd_type
                self._need_admin = need_admin
                self._args = args
                self._func = func
                self._glob = glob
                self._create_pattern()

            def _create_pattern(self):
                self.pattern = '^/' + self._command
                self.pattern += self.command_pattern[self._cmd_type]
                self.help = self.pattern[1:]
                for arg in self._args:
                    self.pattern += self.arg_pattern[arg[0]]
                    if arg[0] != ArgType.ON_OFF:
                        self.help += ' ' + arg[0].name
                if self._glob:
                    self.pattern += ' global'
                    self.help += ' global'
                if self._cmd_type != (ArgType.WORDS and ArgType.STRING):
                    self.pattern += '$'

            @property
            def pattern(self):
                return self._pattern
                
            @pattern.setter
            def pattern(self, value):
                self._pattern = value

            def Do(self, cmd, some_id, admin):
                if (self._need_admin == AdminClass.GLOBAL and
                        admin != AdminClass.GLOBAL):
                    answer = ('To allow this command you need global admin ' +
                        'privileges')
                    return answer, False
                elif (self._need_admin == AdminClass.LOCAL and
                        admin == AdminClass.NONE):
                    answer = 'To allow this command you need admin privileges'
                    return answer, False
                else:
                    args_val = []
                    l = len(self._command)
                    if self._cmd_type == CommandType.INFO:
                        begin = l + 1
                        subcmd = None
                    else:
                        subcmd = cmd[l + 2 : l + 5]
                    if self._cmd_type == CommandType.ADD_DEL:
                        begin = l + 5
                    elif self._cmd_type == CommandType.ON_OFF:
                        begin = l + (4 if cmd[l + 2 : l + 4] == 'on' else 5)
                    elif self._cmd_type == CommandType.SET:
                        begin = l + 5
                        
                    for arg in self._args:
                        if arg[0] != ArgType.ON_OFF:
                            cmd = cmd[begin:]
                            if cmd.startswith(' global'):
                                break
                            reg = re.match(self.arg_pattern[arg[0]], cmd)
                            beg = reg.regs[0][0]
                            val = cmd[beg : beg + reg.regs[0][1]]
                            begin = len(val)
                            val = val[1:]
                            if arg[0] == ArgType.FLOAT:
                                val = float(val)
                            elif arg[0] == ArgType.INTEGER:
                                val = int(val)
                            args_val.append([val, arg[1]])
                        else:
                            args_val.append([
                                    True if subcmd[0:2] == 'on' else False,
                                    arg[1]
                                ])
                    res = None
                    func_args = [
                        a[0] for a in args_val if a[1] == ArgRole.FUNC_ARG
                        ]
                    try:
                        if self._glob:
                            if func_args:
                                res = self._func(*func_args)
                            elif isinstance(self._func, types.FunctionType):
                                res = self._func()
                            else:
                                res = self._func
                        else:
                            if func_args:
                                res = self._func(some_id, *func_args)
                            elif isinstance(self._func, types.FunctionType):
                                res = self._func(some_id)
                            else:
                                res = self._func
                    except Exception:
                        print(traceback.format_exc())
                        return 'Wrond arguments or lambda expression', False

                    if self._cmd_type == CommandType.INFO:
                        save = False
                    else:
                        save = True
                    try:
                        fargs = []
                        for i in range(0, len(args_val)):
                            if args_val[i][1] == ArgRole.ARG:
                                fargs.append(args_val[i][0])
                                if i != len(args_val) - 1:
                                    if args_val[i + 1][1] != ArgRole.ARG:
                                        res = res(*fargs)
                                else:
                                    res = res(*fargs)
                            elif args_val[i][1] == ArgRole.KEY:
                                if i != len(args_val) - 1:
                                    if args_val[i + 1][1] == ArgRole.VALUE:
                                        res[args_val[i][0]] = args_val[i+1][0]
                                        break
                                res = res[args_val[i][0]]
                            elif args_val[i][1] == ArgRole.VALUE:
                                res.val = args_val[i][0]
                                break
                            elif args_val[i][1] == ArgRole.APPEND:
                                have = (True if find_element(
                                        res, lambda n: n == args_val[i][0])
                                    else False)
                                if not have and subcmd == 'add':
                                    res.append(args_val[i][0])
                                elif have and subcmd == 'del':
                                    res.remove(args_val[i][0])
                                elif not have and subcmd == 'del':
                                    return (args_val[i][0] +
                                    ' is not in the list'), False
                                else:
                                    return (args_val[i][0] +
                                    ' already in the list'), False
                                break
                    except Exception:
                        return 'Wrong arguments', False
                    else:
                        if self._cmd_type == CommandType.INFO and res:
                            return res, save
                        else:
                            return 'Success', save


        def __init__(self, command_name, help_):
            self.command = command_name
            self.description = help_
            self._actions = []

        def action(self, cmd_type, need_admin, args, func, glob=False):


            self._actions.append(Control.Command.Action(
                    self.command, cmd_type, need_admin, args, func, glob))
            return self

        def execute(self, command_string, some_id, admin):
            for act in self._actions:
                if re.match(act.pattern, command_string):
                    return act.Do(command_string, some_id, admin)
            return 'Incorrect syntaxis', False

        def get_help(self):
            return [act.help for act in self._actions]


    def __init__(self):
        self._commands = []

    def add(self, command_name, help_):
        new_cmd = Control.Command(command_name, help_)
        self._commands.append(new_cmd)
        return new_cmd

    def execute(self, command_string, some_id, admin):

        if command_string == '/help':
            hlp = ['/' + c.command + '\n' + c.description for c in self._commands]
            hlp.append('/help (command)\nGet help')
            hlp.sort()
            return '\n\n'.join(hlp), False
        elif re.match('^/help ([_a-zа-я]){1,20}$', command_string):
            cmd = find_element(
                    self._commands,
                    lambda c: c.command == command_string[6:]
                )
            if cmd:
                hlp = cmd.description + ':\n\n'
                hlp += '\n'.join(cmd.get_help())
                return hlp, False

        if ' ' in command_string:
            for index in range(1, len(command_string)):
                if command_string[index] == ' ':
                    break
            command_name = command_string[1:index]
        else:
            command_name = command_string[1:]
        cmd = find_element(self._commands, 
                           lambda cmd: cmd.command == command_name)
        if cmd:
            answer, save = cmd.execute(command_string, some_id, admin)
        else:
            answer, save = 'Unknown command', False
        return answer, save




import logging
import logging.handlers
import threading
import json
import re
import traceback
import queue
import sys
import types
import time
import os

import vk_api
from vk_api.longpoll import VkLongPoll, VkEventType

from .dialogthread import DialogThread
from .artem_pb2 import ProtoArtem
from .others import *
from .scenario import Scenario, wrapup
from .cmd import *

VERSION = '1.10.01'
RELEASE = 'Artem3000'

SERIALIZE_FILE = 'dialogs.art'
CHAT_ID_MAX = 10000
DEFAULT_POOLING_INTERVAL = 10.0

if not os.path.exists('log'):
    os.makedirs('log')

ERROR_LOG_FILE = (('log\\' if sys.platform == 'win32' else 'log/') +
                  'artem_' + VERSION + '.log')


class Artem(object):


    def __init__(self, login, password, admins=[], names=[], 
               enabled_session=True, restore=True, twofact_auth=False):

        self._lib = Lib()
        # {some_id: DialogThread}
        self._dialog_threads = {} 
        self._restore = restore
        self._global_admins = admins
        self._secondary_pooling_interval = Wrap()
        self._secondary_pooling_interval.val = DEFAULT_POOLING_INTERVAL
        self._send_queue = queue.Queue()
        
        self._create_logger()
        self._vk_init(login, password, twofact_auth)
        self._control_init()

        response = self._vk.method('users.get')
        name = response[0]['first_name'].lower()
        if name not in names:
            names.append(name)
        name += ' ' + response[0]['last_name'].lower()
        if name not in names:
            names.append(name)
        self._id = response[0]['id']
        self._global_names = sorted(names)

    def on(self, event, handler, prior=0):

        try:
            if isinstance(handler, types.FunctionType):
                handler = type(
                        'Scenario' + str(id(handler)),
                        (Scenario,),
                        {'respond': wrapup(handler)}
                    )
            self._lib.add(event, handler, prior)
            for id_ in self._dialog_threads:
                (self._dialog_threads[id_].lib.add(
                        event, handler, prior))

        except Exception:
            self._logger.error(traceback.format_exc())

    def _create_logger(self):

        self._logger = logging.getLogger()
        self._logger.setLevel(logging.INFO)
        handler = logging.handlers.RotatingFileHandler(
                ERROR_LOG_FILE, maxBytes=1048576, backupCount=5)
        handler.setLevel(logging.INFO)
        formatter = logging.Formatter(
                '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        handler.setFormatter(formatter)
        self._logger.addHandler(handler)

    def _vk_init(self, login, password, twofact_auth):

        try:
            if not twofact_auth:
                self._vk = vk_api.VkApi(login, password)
            else:
                self._vk = vk_api.VkApi(
                        login, password, 
                        auth_handler = lambda: 
                        (input('Enter authentication code: '), True)
                    )
            self._vk.auth()

        except Exception:
            self._logger.error(traceback.format_exc())

    def _get_interlocutors(self, some_id): 

            if some_id < CHAT_ID_MAX:
                response = self._vk.method(
                        'messages.getChat',
                        {'chat_id': some_id, 'fields': 'users'})
                inters = [Interlocutor(
                                user['id'], 
                                user['first_name'], 
                                user['last_name']) 
                            for user in response['users']
                            if user['id'] != self._id
                         ]
            else:
                response = self._vk.method(
                        'users.get', 
                        {'user_ids': some_id})
                inters = [Interlocutor(
                            some_id,
                            response[0]['first_name'],
                            response[0]['last_name']
                         )]
            return inters

    def _create_dialog_thread(self, some_id, start=False):

        if some_id not in self._dialog_threads:
            dialog_thread = DialogThread(
                    some_id, self._send_queue, self._lib,
                    self._get_interlocutors(some_id), self._logger,
                    start, self._global_names, self._global_admins
                )
            dialog_thread.start()
            self._dialog_threads[some_id] = dialog_thread
            if not start and self._restore:
                self._serialize()

    def _serialize(self):
        try:
            art = ProtoArtem()
            for admin in self._global_admins:
                art.global_admins.append(admin)
            for name in self._global_names:
                art.global_names.append(name)
            art.pooling_interval = self._secondary_pooling_interval.val
            for id_ in self._dialog_threads:
                thr = art.dialog_threads.add()
                thr.some_id = id_
                thr.discourse_interval_max = (
                        self._dialog_threads[id_].discourse_interval_max.val)
                thr.session_duration = (
                        self._dialog_threads[id_].session_duration.val)
                thr.sessions = self._dialog_threads[id_].enabled_session.val
                for name in self._dialog_threads[id_].local_names:
                    thr.names.append(name)
                for admin in self._dialog_threads[id_].local_admins:
                    thr.names.append(admin)

            with open(SERIALIZE_FILE, 'wb') as protobuf_file:
                protobuf_file.write(art.SerializeToString())

        except Exception:
            self._logger.error(traceback.format_exc())

    def _deserialize(self):
        try:
            art = ProtoArtem()
            with open(SERIALIZE_FILE, 'rb') as protobuf_file:
                art.ParseFromString(protobuf_file.read())

            self._global_admins = [admin for admin in art.global_admins]
            self._global_names = [name for name in art.global_names]
            self._secondary_pooling_interval.val = art.pooling_interval
            for thr in art.dialog_threads:
                self._create_dialog_thread(thr.some_id, start=True)
                self._dialog_threads[thr.some_id].restore(
                        thr.sessions, thr.session_duration,
                        thr.discourse_interval_max,
                        [name for name in thr.names], 
                        [admin for admin in thr.admins]
                    )
        except FileNotFoundError:
            pass
        except Exception:
            self._logger.error(traceback.format_exc())

    def _send_listener(self):
        while True:
            try:
                answer = self._send_queue.get()
                if answer.id < CHAT_ID_MAX:
                    self._vk.method(
                            'messages.send',
                            {'chat_id': answer.id, 
                            'message': answer.message,
                            'attachment': answer.attach}
                            )
                else:
                    self._vk.method(
                            'messages.send', 
                            {'user_id': answer.id, 
                            'message': answer.message,
                            'attachment': answer.attach}
                            )

            except Exception:
                self._logger.error(traceback.format_exc())

    def _newfriend_pooling(self):

        while True:
            try:
                response = self._vk.method(
                        'friends.getRequests',
                        {'count': 100, 'out': 0,
                        'extended': 1, 'need_viewed': 1}
                        )
                if response['count'] != 0:
                    for item in response['items']:
                        self._vk.method(
                                'friends.add', 
                                {'user_id': item['user_id'], 
                                'follow': 0}
                                )
                        if item['user_id'] not in self._dialog_threads:
                            self._create_dialog_thread(item['user_id'])
                        (self._dialog_threads[item['user_id']]
                                .queue.put(Envelope(
                                    Event.ADDFRIEND,
                                    item['user_id'], None)
                                ))
                time.sleep(self._secondary_pooling_interval.val)

            except Exception:
                self._logger.error(traceback.format_exc())

    def alive(self):

        threading.Thread(target=self._send_listener).start()
        threading.Thread(target=self._newfriend_pooling).start()
        if self._restore:
            self._deserialize()

        while True:
            try:
                longpool = VkLongPoll(self._vk)
                for event in longpool.listen():
                    if (event.type == VkEventType.MESSAGE_NEW and
                            not event.from_me):

                        some_id = (event.chat_id if event.from_chat
                                else event.user_id)
                        if some_id not in self._dialog_threads:
                            self._create_dialog_thread(some_id)

                        if event.text.startswith('/'):
                            self._commands(event.text.lower(), 
                                           event.user_id, some_id)
                        else:
                            (self._dialog_threads[some_id].
                                    queue.put(Envelope(
                                        Event.ANSWER,
                                        event.user_id, event.text.lower())
                                    ))
            except Exception:
                self._logger.error(traceback.format_exc())

    def _commands(self, message, user_id, some_id):
        if user_id in self._global_admins:
            admin = AdminClass.GLOBAL
        elif user_id in self._dialog_threads[some_id].local_admins:
            admin = AdminClass.LOCAL
        else:
            admin = AdminClass.NONE
        answer, need_save = self._cmd.execute(message, some_id, admin)
        if need_save:
            self._serialize()
        self._send_queue.put(ToSend(some_id, answer, None))

    def _control_init(self):
        self._cmd = Control()
        self._cmd.add('version', 'Information about Artem version'
            ).action(
                CommandType.INFO, 
                AdminClass.NONE, 
                [],
                lambda some_id:
                    'ArtemCore v' + VERSION + '\nRelease: ' + RELEASE
            )
        self._cmd.add('scenario', 'Enable or disable a specific scenario'
            ).action(
                CommandType.INFO,
                AdminClass.LOCAL,
                [[ArgType.WORD, ArgRole.FUNC_ARG],
                 [ArgType.WORD, ArgRole.FUNC_ARG]],
                lambda some_id, *args: 
                    ('Local scenario status: ' + 
                    ('ON' if self._dialog_threads[some_id].lib.get_status(args[0], args[1])
                          else 'OFF'))
            ).action(
                CommandType.INFO,
                AdminClass.LOCAL,
                [[ArgType.WORD, ArgRole.FUNC_ARG],
                 [ArgType.WORD, ArgRole.FUNC_ARG]],
                lambda *args:
                    ('Global scenario status: ' + 
                    ('ON' if self._lib.get_status(args[0], args[1]) else 'OFF')),
                glob=True
            ).action(
                CommandType.ON_OFF,
                AdminClass.LOCAL,
                [[ArgType.WORD, ArgRole.ARG],
                 [ArgType.WORD, ArgRole.ARG],
                 [ArgType.ON_OFF, ArgRole.ARG]],
                lambda some_id:
                    self._dialog_threads[some_id].lib.set_status
            ).action(
                CommandType.ON_OFF,
                AdminClass.GLOBAL,
                [[ArgType.WORD, ArgRole.ARG],
                 [ArgType.WORD, ArgRole.ARG],
                 [ArgType.ON_OFF, ArgRole.ARG]],
                lambda:
                    self._lib.set_status,
                glob=True
            )
        self._cmd.add('pooling_interval',
            'Get or set time between secondary pooling (all except messages)'
            ).action(
                CommandType.INFO,
                AdminClass.NONE,
                [],
                lambda some_id:
                    ('Secondary pooling interval = ' +
                        str(self._secondary_pooling_interval.val) + ' sec')
            ).action(
                CommandType.SET,
                AdminClass.GLOBAL,
                [[ArgType.FLOAT, ArgRole.VALUE]],
                lambda some_id:
                    self._secondary_pooling_interval
            )
        self._cmd.add('admins',
            'Information and administration of a group of admins'
            ).action(
                CommandType.INFO,
                AdminClass.NONE,
                [],
                lambda some_id:
                    ('Local admins: ' + ', '.join(
                        [str(a) for a in self._dialog_threads[some_id].local_admins]))
            ).action(
                CommandType.INFO,
                AdminClass.NONE,
                [],
                lambda:
                    ('Global admins: ' + ', '.join(
                        [str(a) for a in self._global_admins])),
                glob=True
            ).action(
                CommandType.ADD_DEL,
                AdminClass.LOCAL,
                [[ArgType.INTEGER, ArgRole.APPEND]],
                lambda some_id:
                    self._dialog_threads[some_id].local_admins
            ).action(
                CommandType.ADD_DEL,
                AdminClass.GLOBAL,
                [[ArgType.INTEGER, ArgRole.APPEND]],
                lambda:
                    self._global_admins,
                glob=True
            )
        self._cmd.add('names',
            'Information and administration of Artem respond names'
            ).action(
                CommandType.INFO,
                AdminClass.NONE,
                [],
                lambda some_id:
                    ('Artem local names: ' + ', '.join(
                        [str(a) for a in self._dialog_threads[some_id].local_names]))
            ).action(
                CommandType.INFO,
                AdminClass.NONE,
                [],
                lambda:
                    ('Artem global names: ' + ', '.join(
                        [str(a) for a in self._global_names])),
                glob=True
            ).action(
                CommandType.ADD_DEL,
                AdminClass.LOCAL,
                [[ArgType.STRING, ArgRole.APPEND]],
                lambda some_id:
                    self._dialog_threads[some_id].local_names
            ).action(
                CommandType.ADD_DEL,
                AdminClass.GLOBAL,
                [[ArgType.STRING, ArgRole.APPEND]],
                lambda:
                    self._global_names,
                glob=True
            )
        self._cmd.add('sessions', 'Enabled or disabled sessions'
            ).action(
                CommandType.INFO,
                AdminClass.NONE,
                [],
                lambda some_id:
                    'Sessions ' + ('enabled'
                        if self._dialog_threads[some_id].enabled_session.val
                        else 'disabled') +' in local chat'
            ).action(
                CommandType.ON_OFF,
                AdminClass.LOCAL,
                [[ArgType.ON_OFF, ArgRole.VALUE]],
                lambda some_id:
                    self._dialog_threads[some_id].enabled_session
            )
        self._cmd.add('session_duration', 'Get or set duration of session'
            ).action(
                CommandType.INFO,
                AdminClass.NONE,
                [],
                lambda some_id:
                    ('Session duration = ' +
                        str(self._dialog_threads[some_id].session_duration.val) + ' sec')
            ).action(
                CommandType.SET,
                AdminClass.GLOBAL,
                [[ArgType.FLOAT, ArgRole.VALUE]],
                lambda some_id:
                    self._dialog_threads[some_id].session_duration
            )
        self._cmd.add('discourse_interval',
            'Get or set maximal time between two discourse'
            ).action(
                CommandType.INFO,
                AdminClass.NONE,
                [],
                lambda some_id:
                    ('Max discourse interval = ' +
                        str(self._dialog_threads[some_id].discourse_interval_max.val) + ' sec')
            ).action(
                CommandType.SET,
                AdminClass.GLOBAL,
                [[ArgType.FLOAT, ArgRole.VALUE]],
                lambda some_id:
                    self._dialog_threads[some_id].discourse_interval_max
            )
        self._cmd.add('events',
            'Get information about event types and their scenarios'
            ).action(
                CommandType.INFO,
                AdminClass.LOCAL,
                [],
                lambda some_id:
                    ', '.join([(e.name + ': ' + str(len(self._lib[e])))
                            for e in Event])
            ).action(
                CommandType.INFO,
                AdminClass.LOCAL,
                [[ArgType.WORD, ArgRole.FUNC_ARG]],
                lambda some_id, *args:
                    args[0].upper() + ' scenarios:\n' +
                        '\n'.join(p.scn_type.__name__
                                  for p in self._lib[args[0]])
            )
        self._cmd.add('dialogs',
            'Provide information about artem\'s dialogs'
            ).action(
                CommandType.INFO,
                AdminClass.GLOBAL,
                [],
                lambda some_id:
                    'Chats with Artem: ' + ', '.join(
                        str(id_) for id_ in self._dialog_threads)
            ).action(
                CommandType.INFO,
                AdminClass.GLOBAL,
                [[ArgType.INTEGER, ArgRole.FUNC_ARG]],
                lambda some_id, *args:
                    ('Dialog ' + str(args[0]) + ' interlocutors:\n' +
                        '\n'.join(
                            (str(i.id) + ' - ' + i.first_name + ' ' + i.last_name)
                            for i in self._dialog_threads[args[0]].interlocutors
                        )
                    )
            )
        self._cmd.add('id', 'Get id of current Artem account'
            ).action(
                CommandType.INFO,
                AdminClass.LOCAL,
                [],
                lambda some_id:
                    'ID of current Artem:  ' + str(self._id)
            )
        self._cmd.add('sendto', 'Forcing a message to the specified id'
            ).action(
                CommandType.INFO,
                AdminClass.GLOBAL,
                [[ArgType.INTEGER, ArgRole.FUNC_ARG],
                 [ArgType.MESSAGE, ArgRole.FUNC_ARG]],
                lambda some_id, *args:
                    self._send_queue.put(ToSend(
                        args[0], args[1][0].upper() + args[1][1:], None))
            )

        



from os.path import realpath, dirname, join
from setuptools import setup, find_packages

import artem

VERSION = artem.artem_core.VERSION
PROJECT_ROOT = dirname(realpath(__file__))

REQUIREMENTS_FILE = join(PROJECT_ROOT, 'requirements.txt')

with open(REQUIREMENTS_FILE) as f:
    install_reqs = f.read().splitlines()

install_reqs.append('setuptools')


setup(name='artem',
      version=VERSION,
      description='Simple core for creating chat bots in VK.com',
      author='ParadoxMaster',
      author_email='Chesh397@mail.ru',
      url='https://github.com/Tgjmjgj/artem',
      download_url = ('https://github.com/Tgjmjgj/artem/archive/' + 
            artem.artem_core.VERSION + '.tar.gz'),
      packages=find_packages(),
      include_package_data=True,
      install_requires=install_reqs,
      extras_require={},
      license='Apache 2.0',
      platforms='any',
      keywords=['vk','chat bot', 'bot core'],
      classifiers=[
            'License :: OSI Approved :: Apache Software License',
            'Development Status :: 3 - Alpha',
            'Environment :: Console',
            'Intended Audience :: Developers',
            'Operating System :: Microsoft :: Windows',
            'Operating System :: Unix',
            'Programming Language :: Python :: 3 :: Only',
            'Programming Language :: Python :: 3.6',
            'Topic :: Scientific/Engineering :: Artificial Intelligence',
            'Topic :: Communications :: Chat'
      ],
      long_description="""
Simple core for creating chat bots in VK.com
=============
eMail: Chesh397@mail.ru
This project is hosted at https://github.com/Tgjmjgj/artem
""")